/* ###*B*###
 * ERIKA Enterprise - a tiny RTOS for small microcontrollers
 *
 * Copyright (C) 2002-2012  Evidence Srl
 *
 * This file is part of ERIKA Enterprise.
 *
 * ERIKA Enterprise is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation,
 * (with a special exception described below).
 *
 * Linking this code statically or dynamically with other modules is
 * making a combined work based on this code.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this code with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on this library.  If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * ERIKA Enterprise is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License version 2 for more details.
 *
 * You should have received a copy of the GNU General Public License
 * version 2 along with ERIKA Enterprise; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 * ###*E*### */

 /** 
	
	@file ee_isr.h
	@brief Interrupt Services Routines
	@author Gianluca Franchino
	@date 2012
	@author Giuseppe Serano
	@date 2013
	Based on mcu/ti_stellaris_lm4f232xxxx/inc/ee_isr.h
*/ 

#ifndef __INCLUDE_RENESAS_R5F5210X_ISR_H__
#define __INCLUDE_RENESAS_R5F5210X_ISR_H__

#include "ee.h"

#ifdef __CCRX__
#include "cpu/common/inc/ee_compiler_ccrx.h"
#else
#error Unsupported compiler
#endif

/** @brief	ISRs Initialization. */
void EE_rx200_init_isr(void);

/** Bus-error interrupt */
#ifdef	EE_RX200_BSC_BUSERR_ISR
#ifdef	EE_RX200_BSC_BUSERR_ISR_PRI
#define	EE_RX200_ICU_IPR_BSC_BUSERR	EE_HWREG8(0x00087300U)
#endif	/* EE_RX200_BSC_BUSERR_ISR_PRI */
#define	EE_RX200_ICU_IER_BSC_BUSERR	EE_HWREG8(0x00087202U)
#define	EE_RX200_ICU_BSC_BUSERR_MASK	0x00000001U
#endif	/* EE_RX200_BSC_BUSERR_ISR */

/** Flash interface error interrupt. */
#ifdef	EE_RX200_FCU_FIFERR_ISR
#ifdef	EE_RX200_FCU_FIFERR_ISR_PRI
#define	EE_RX200_ICU_IPR_FCU_FIFERR	EE_HWREG8(0x00087301U)
#endif	/* EE_RX200_FCU_FIFERR_ISR_PRI */
#define	EE_RX200_ICU_IER_FCU_FIFERR	EE_HWREG8(0x00087202U)
#define	EE_RX200_ICU_FCU_FIFERR_MASK	0x00000020U
#endif	/* EE_RX200_FCU_FIFERR_ISR */

/** Flash READY interrupt. */
#ifdef	EE_RX200_FCU_FRDYI_ISR
#ifdef	EE_RX200_FCU_FRDYI_ISR_PRI
#define	EE_RX200_ICU_IPR_FCU_FRDYI	EE_HWREG8(0x00087302U)
#endif	/* EE_RX200_FCU_FRDYI_ISR_PRI */
#define	EE_RX200_ICU_IER_FCU_FRDYI	EE_HWREG8(0x00087202U)
#define	EE_RX200_ICU_FCU_FRDYI_MASK	0x00000080U
#endif	/* EE_RX200_FCU_FRDYI_ISR */

/** Software interrupt. */
#ifdef	EE_RX200_ICU_SWINT_ISR
#ifdef	EE_RX200_ICU_SWINT_ISR_PRI
#define	EE_RX200_ICU_IPR_ICU_SWINT	EE_HWREG8(0x00087303U)
#endif	/* EE_RX200_ICU_SWINT_ISR_PRI */
#define	EE_RX200_ICU_IER_ICU_SWINT	EE_HWREG8(0x00087203U)
#define	EE_RX200_ICU_ICU_SWINT_MASK	0x00000008U
#endif	/* EE_RX200_ICU_SWINT_ISR */

/** Compare Match 0 interrupt. */
#ifdef	EE_RX200_CMT0_CMI0_ISR
#ifdef	EE_RX200_CMT0_CMI0_ISR_PRI
#define	EE_RX200_ICU_IPR_CMT0_CMI0	EE_HWREG8(0x00087304U)
#endif	/* EE_RX200_CMT0_CMI0_ISR_PRI */
#define	EE_RX200_ICU_IER_CMT0_CMI0	EE_HWREG8(0x00087203U)
#define	EE_RX200_ICU_CMT0_CMI0_MASK	0x00000010U
#endif	/* EE_RX200_CMT0_CMI0_ISR */

/** Compare Match 1 interrupt. */
#ifdef	EE_RX200_CMT1_CMI1_ISR
#ifdef	EE_RX200_CMT1_CMI1_ISR_PRI
#define	EE_RX200_ICU_IPR_CMT1_CMI1	EE_HWREG8(0x00087305U)
#endif	/* EE_RX200_CMT1_CMI1_ISR_PRI */
#define	EE_RX200_ICU_IER_CMT1_CMI1	EE_HWREG8(0x00087203U)
#define	EE_RX200_ICU_CMT1_CMI1_MASK	0x00000020U
#endif	/* EE_RX200_CMT1_CMI1_ISR */

/** Compare Match 2 interrupt. */
#ifdef	EE_RX200_CMT2_CMI2_ISR
#ifdef	EE_RX200_CMT2_CMI2_ISR_PRI
#define	EE_RX200_ICU_IPR_CMT2_CMI2	EE_HWREG8(0x00087306U)
#endif	/* EE_RX200_CMT2_CMI2_ISR_PRI */
#define	EE_RX200_ICU_IER_CMT2_CMI2	EE_HWREG8(0x00087203U)
#define	EE_RX200_ICU_CMT2_CMI2_MASK	0x00000040U
#endif	/* EE_RX200_CMT2_CMI2_ISR */

/** Compare Match 3 interrupt. */
#ifdef	EE_RX200_CMT3_CMI3_ISR
#ifdef	EE_RX200_CMT3_CMI3_ISR_PRI
#define	EE_RX200_ICU_IPR_CMT3_CMI3	EE_HWREG8(0x00087307U)
#endif	/* EE_RX200_CMT3_CMI3_ISR_PRI */
#define	EE_RX200_ICU_IER_CMT3_CMI3	EE_HWREG8(0x00087203U)
#define	EE_RX200_ICU_CMT3_CMI3_MASK	0x00000080U
#endif	/* EE_RX200_CMT3_CMI3_ISR */

/** Frequency Error interrupt. */
#ifdef	EE_RX200_CAC_FERRF_ISR
#ifdef	EE_RX200_CAC_FERRF_ISR_PRI
#define	EE_RX200_ICU_IPR_CAC_FERRF	EE_HWREG8(0x00087320U)
#endif	/* EE_RX200_CAC_FERRF_ISR_PRI */
#define	EE_RX200_ICU_IER_CAC_FERRF	EE_HWREG8(0x00087204U)
#define	EE_RX200_ICU_CAC_FERRF_MASK	0x00000001U
#endif	/* EE_RX200_CAC_FERRF_ISR */

/** Measurement End interrupt. */
#ifdef	EE_RX200_CAC_MENDF_ISR
#ifdef	EE_RX200_CAC_MENDF_ISR_PRI
#define	EE_RX200_ICU_IPR_CAC_MENDF	EE_HWREG8(0x00087321U)
#endif	/* EE_RX200_CAC_MENDF_ISR_PRI */
#define	EE_RX200_ICU_IER_CAC_MENDF	EE_HWREG8(0x00087204U)
#define	EE_RX200_ICU_CAC_MENDF_MASK	0x00000002U
#endif	/* EE_RX200_CAC_MENDF_ISR */

/** Overflow interrupt. */
#ifdef	EE_RX200_CAC_OVFF_ISR
#ifdef	EE_RX200_CAC_OVFF_ISR_PRI
#define	EE_RX200_ICU_IPR_CAC_OVFF	EE_HWREG8(0x00087322U)
#endif	/* EE_RX200_CAC_OVFF_ISR_PRI */
#define	EE_RX200_ICU_IER_CAC_OVFF	EE_HWREG8(0x00087204U)
#define	EE_RX200_ICU_CAC_OVFF_MASK	0x00000004U
#endif	/* EE_RX200_CAC_OVFF_ISR */

/** RSPI Errors interrupt. */
#ifdef	EE_RX200_RSPI0_SPEI0_ISR
#ifdef	EE_RX200_RSPI0_SPEI0_ISR_PRI
#define	EE_RX200_ICU_IPR_RSPI0_SPEI0	EE_HWREG8(0x0008732CU)
#endif	/* EE_RX200_RSPI0_SPEI0_ISR_PRI */
#define	EE_RX200_ICU_IER_RSPI0_SPEI0	EE_HWREG8(0x00087205U)
#define	EE_RX200_ICU_RSPI0_SPEI0_MASK	0x00000010U
#endif	/* EE_RX200_RSPI0_SPEI0_ISR */

/** Receive Buffer Full interrupt. */
#ifdef	EE_RX200_RSPI0_SPRI0_ISR
#ifdef	EE_RX200_RSPI0_SPRI0_ISR_PRI
#define	EE_RX200_ICU_IPR_RSPI0_SPRI0	EE_HWREG8(0x0008732CU)
#endif	/* EE_RX200_RSPI0_SPRI0_ISR_PRI */
#define	EE_RX200_ICU_IER_RSPI0_SPRI0	EE_HWREG8(0x00087205U)
#define	EE_RX200_ICU_RSPI0_SPRI0_MASK	0x00000020U
#endif	/* EE_RX200_RSPI0_SPRI0_ISR */

/** Transmit Buffer Empty interrupt. */
#ifdef	EE_RX200_RSPI0_SPTI0_ISR
#ifdef	EE_RX200_RSPI0_SPTI0_ISR_PRI
#define	EE_RX200_ICU_IPR_RSPI0_SPTI0	EE_HWREG8(0x0008732CU)
#endif	/* EE_RX200_RSPI0_SPTI0_ISR_PRI */
#define	EE_RX200_ICU_IER_RSPI0_SPTI0	EE_HWREG8(0x00087205U)
#define	EE_RX200_ICU_RSPI0_SPTI0_MASK	0x00000040U
#endif	/* EE_RX200_RSPI0_SPTI0_ISR */

/** RSPI Idle interrupt. */
#ifdef	EE_RX200_RSPI0_SPII0_ISR
#ifdef	EE_RX200_RSPI0_SPII0_ISR_PRI
#define	EE_RX200_ICU_IPR_RSPI0_SPII0	EE_HWREG8(0x0008732CU)
#endif	/* EE_RX200_RSPI0_SPII0_ISR_PRI */
#define	EE_RX200_ICU_IER_RSPI0_SPII0	EE_HWREG8(0x00087205U)
#define	EE_RX200_ICU_RSPI0_SPII0_MASK	0x00000080U
#endif	/* EE_RX200_RSPI0_SPII0_ISR */

/** Data Operation Circuit interrupt. */
#ifdef	EE_RX200_DOC_DOPCF_ISR
#ifdef	EE_RX200_DOC_DOPCF_ISR_PRI
#define	EE_RX200_ICU_IPR_DOC_DOPCF	EE_HWREG8(0x00087339U)
#endif	/* EE_RX200_DOC_DOPCF_ISR_PRI */
#define	EE_RX200_ICU_IER_DOC_DOPCF	EE_HWREG8(0x00087207U)
#define	EE_RX200_ICU_DOC_DOPCF_MASK	0x00000002U
#endif	/* EE_RX200_DOC_DOPCF_ISR */

/** Comparator B0 interrupt. */
#ifdef	EE_RX200_CMPB_CMPB0_ISR
#ifdef	EE_RX200_CMPB_CMPB0_ISR_PRI
#define	EE_RX200_ICU_IPR_CMPB_CMPB0	EE_HWREG8(0x0008733AU)
#endif	/* EE_RX200_CMPB_CMPB0_ISR_PRI */
#define	EE_RX200_ICU_IER_CMPB_CMPB0	EE_HWREG8(0x00087207U)
#define	EE_RX200_ICU_CMPB_CMPB0_MASK	0x00000004U
#endif	/* EE_RX200_CMPB_CMPB0_ISR */

/** Comparator B1 interrupt. */
#ifdef	EE_RX200_CMPB_CMPB1_ISR
#ifdef	EE_RX200_CMPB_CMPB1_ISR_PRI
#define	EE_RX200_ICU_IPR_CMPB_CMPB1	EE_HWREG8(0x0008733BU)
#endif	/* EE_RX200_CMPB_CMPB1_ISR_PRI */
#define	EE_RX200_ICU_IER_CMPB_CMPB1	EE_HWREG8(0x00087207U)
#define	EE_RX200_ICU_CMPB_CMPB1_MASK	0x00000008U
#endif	/* EE_RX200_CMPB_CMPB1_ISR */

/** RTC Count Up interrupt. */
#ifdef	EE_RX200_RTC_CUP_ISR
#ifdef	EE_RX200_RTC_CUP_ISR_PRI
#define	EE_RX200_ICU_IPR_RTC_CUP	EE_HWREG8(0x0008733FU)
#endif	/* EE_RX200_RTC_CUP_ISR_PRI */
#define	EE_RX200_ICU_IER_RTC_CUP	EE_HWREG8(0x00087207U)
#define	EE_RX200_ICU_RTC_CUP_MASK	0x00000080U
#endif	/* EE_RX200_RTC_CUP_ISR */

/** IRQ0 interrupt. */
#ifdef	EE_RX200_ICU_IRQ0_ISR
#ifdef	EE_RX200_ICU_IRQ0_ISR_PRI
#define	EE_RX200_ICU_IPR_ICU_IRQ0	EE_HWREG8(0x00087340U)
#endif	/* EE_RX200_ICU_IRQ0_ISR_PRI */
#define	EE_RX200_ICU_IER_ICU_IRQ0	EE_HWREG8(0x00087208U)
#define	EE_RX200_ICU_ICU_IRQ0_MASK	0x00000001U
#endif	/* EE_RX200_ICU_IRQ0_ISR */

/** IRQ1 interrupt. */
#ifdef	EE_RX200_ICU_IRQ1_ISR
#ifdef	EE_RX200_ICU_IRQ1_ISR_PRI
#define	EE_RX200_ICU_IPR_ICU_IRQ1	EE_HWREG8(0x00087341U)
#endif	/* EE_RX200_ICU_IRQ1_ISR_PRI */
#define	EE_RX200_ICU_IER_ICU_IRQ1	EE_HWREG8(0x00087208U)
#define	EE_RX200_ICU_ICU_IRQ1_MASK	0x00000002U
#endif	/* EE_RX200_ICU_IRQ1_ISR */

/** IRQ2 interrupt. */
#ifdef	EE_RX200_ICU_IRQ2_ISR
#ifdef	EE_RX200_ICU_IRQ2_ISR_PRI
#define	EE_RX200_ICU_IPR_ICU_IRQ2	EE_HWREG8(0x00087342U)
#endif	/* EE_RX200_ICU_IRQ2_ISR_PRI */
#define	EE_RX200_ICU_IER_ICU_IRQ2	EE_HWREG8(0x00087208U)
#define	EE_RX200_ICU_ICU_IRQ2_MASK	0x00000004U
#endif	/* EE_RX200_ICU_IRQ2_ISR */

/** IRQ3 interrupt. */
#ifdef	EE_RX200_ICU_IRQ3_ISR
#ifdef	EE_RX200_ICU_IRQ3_ISR_PRI
#define	EE_RX200_ICU_IPR_ICU_IRQ3	EE_HWREG8(0x00087343U)
#endif	/* EE_RX200_ICU_IRQ3_ISR_PRI */
#define	EE_RX200_ICU_IER_ICU_IRQ3	EE_HWREG8(0x00087208U)
#define	EE_RX200_ICU_ICU_IRQ3_MASK	0x00000008U
#endif	/* EE_RX200_ICU_IRQ3_ISR */

/** IRQ4 interrupt. */
#ifdef	EE_RX200_ICU_IRQ4_ISR
#ifdef	EE_RX200_ICU_IRQ4_ISR_PRI
#define	EE_RX200_ICU_IPR_ICU_IRQ4	EE_HWREG8(0x00087344U)
#endif	/* EE_RX200_ICU_IRQ4_ISR_PRI */
#define	EE_RX200_ICU_IER_ICU_IRQ4	EE_HWREG8(0x00087208U)
#define	EE_RX200_ICU_ICU_IRQ4_MASK	0x00000010U
#endif	/* EE_RX200_ICU_IRQ4_ISR */

/** IRQ5 interrupt. */
#ifdef	EE_RX200_ICU_IRQ5_ISR
#ifdef	EE_RX200_ICU_IRQ5_ISR_PRI
#define	EE_RX200_ICU_IPR_ICU_IRQ5	EE_HWREG8(0x00087345U)
#endif	/* EE_RX200_ICU_IRQ5_ISR_PRI */
#define	EE_RX200_ICU_IER_ICU_IRQ5	EE_HWREG8(0x00087208U)
#define	EE_RX200_ICU_ICU_IRQ5_MASK	0x00000020U
#endif	/* EE_RX200_ICU_IRQ5_ISR */

/** IRQ6 interrupt. */
#ifdef	EE_RX200_ICU_IRQ6_ISR
#ifdef	EE_RX200_ICU_IRQ6_ISR_PRI
#define	EE_RX200_ICU_IPR_ICU_IRQ6	EE_HWREG8(0x00087346U)
#endif	/* EE_RX200_ICU_IRQ6_ISR_PRI */
#define	EE_RX200_ICU_IER_ICU_IRQ6	EE_HWREG8(0x00087208U)
#define	EE_RX200_ICU_ICU_IRQ6_MASK	0x00000040U
#endif	/* EE_RX200_ICU_IRQ6_ISR */

/** IRQ7 interrupt. */
#ifdef	EE_RX200_ICU_IRQ7_ISR
#ifdef	EE_RX200_ICU_IRQ7_ISR_PRI
#define	EE_RX200_ICU_IPR_ICU_IRQ7	EE_HWREG8(0x00087347U)
#endif	/* EE_RX200_ICU_IRQ7_ISR_PRI */
#define	EE_RX200_ICU_IER_ICU_IRQ7	EE_HWREG8(0x00087208U)
#define	EE_RX200_ICU_ICU_IRQ7_MASK	0x00000080U
#endif	/* EE_RX200_ICU_IRQ7_ISR */

/** LDV1/CMPA1 interrupt. */
#ifdef	EE_RX200_LVD1_CMPA1_ISR
#ifdef	EE_RX200_LVD1_CMPA1_ISR_PRI
#define	EE_RX200_ICU_IPR_LVD1_CMPA1	EE_HWREG8(0x00087358U)
#endif	/* EE_RX200_LVD1_CMPA1_ISR_PRI */
#define	EE_RX200_ICU_IER_LVD1_CMPA1	EE_HWREG8(0x0008720BU)
#define	EE_RX200_ICU_LVD1_CMPA1_MASK	0x00000001U
#endif	/* EE_RX200_LVD1_CMPA1_ISR */

/** LDV2/CMPA2 interrupt. */
#ifdef	EE_RX200_LVD2_CMPA2_ISR
#ifdef	EE_RX200_LVD2_CMPA2_ISR_PRI
#define	EE_RX200_ICU_IPR_LVD2_CMPA2	EE_HWREG8(0x00087359U)
#endif	/* EE_RX200_LVD2_CMPA2_ISR_PRI */
#define	EE_RX200_ICU_IER_LVD2_CMPA2	EE_HWREG8(0x0008720BU)
#define	EE_RX200_ICU_LVD2_CMPA2_MASK	0x00000002U
#endif	/* EE_RX200_LVD2_CMPA2_ISR */

/** RTC Alarm interrupt. */
#ifdef	EE_RX200_RTC_ALM_ISR
#ifdef	EE_RX200_RTC_ALM_ISR_PRI
#define	EE_RX200_ICU_IPR_RTC_ALM	EE_HWREG8(0x0008735CU)
#endif	/* EE_RX200_LVD2_CMPA1_ISR_PRI */
#define	EE_RX200_ICU_IER_RTC_ALM	EE_HWREG8(0x0008720BU)
#define	EE_RX200_ICU_RTC_ALM_MASK	0x00000010U
#endif	/* EE_RX200_RTC_ALM_ISR */

/** RTC Periodic interrupt. */
#ifdef	EE_RX200_RTC_PRD_ISR
#ifdef	EE_RX200_RTC_PRD_ISR_PRI
#define	EE_RX200_ICU_IPR_RTC_PRD	EE_HWREG8(0x0008735DU)
#endif	/* EE_RX200_RTC_PRD_ISR_PRI */
#define	EE_RX200_ICU_IER_RTC_PRD	EE_HWREG8(0x0008720BU)
#define	EE_RX200_ICU_RTC_PRD_MASK	0x00000020U
#endif	/* EE_RX200_RTC_PRD_ISR */

/** ADC Scan End interrupt. */
#ifdef	EE_RX200_S12AD_S12ADI0_ISR
#ifdef	EE_RX200_S12AD_S12ADI0_ISR_PRI
#define	EE_RX200_ICU_IPR_S12AD_S12ADI0	EE_HWREG8(0x00087366U)
#endif	/* EE_RX200_S12AD_S12ADI0_ISR_PRI */
#define	EE_RX200_ICU_IER_S12AD_S12ADI0	EE_HWREG8(0x0008720CU)
#define	EE_RX200_ICU_S12AD_S12ADI0_MASK	0x00000040U
#endif	/* EE_RX200_S12AD_S12ADI0_ISR */

/** ADC Group B Scan End interrupt. */
#ifdef	EE_RX200_S12AD_GBADI_ISR
#ifdef	EE_RX200_S12AD_GBADI_ISR_PRI
#define	EE_RX200_ICU_IPR_S12AD_GBADI	EE_HWREG8(0x00087367U)
#endif	/* EE_RX200_S12AD_GBADI_ISR_PRI */
#define	EE_RX200_ICU_IER_S12AD_GBADI	EE_HWREG8(0x0008720CU)
#define	EE_RX200_ICU_S12AD_GBADI_MASK	0x00000080U
#endif	/* EE_RX200_S12AD_GBADI_ISR */

/** Event Link SR18 interrupt. */
#ifdef	EE_RX200_ELC_ELSR18I_ISR
#ifdef	EE_RX200_ELC_ELSR18I_ISR_PRI
#define	EE_RX200_ICU_IPR_ELC_ELSR18I	EE_HWREG8(0x0008736AU)
#endif	/* EE_RX200_ELC_ELSR18I_ISR_PRI */
#define	EE_RX200_ICU_IER_ELC_ELSR18I	EE_HWREG8(0x0008720DU)
#define	EE_RX200_ICU_ELC_ELSR18I_MASK	0x00000004U
#endif	/* EE_RX200_ELC_ELSR18I_ISR */

/** Event Link SR19 interrupt. */
#ifdef	EE_RX200_ELC_ELSR19I_ISR
#ifdef	EE_RX200_ELC_ELSR19I_ISR_PRI
#define	EE_RX200_ICU_IPR_ELC_ELSR19I	EE_HWREG8(0x0008736BU)
#endif	/* EE_RX200_ELC_ELSR19I_ISR_PRI */
#define	EE_RX200_ICU_IER_ELC_ELSR19I	EE_HWREG8(0x0008720DU)
#define	EE_RX200_ICU_ELC_ELSR19I_MASK	0x00000008U
#endif	/* EE_RX200_ELC_ELSR19I_ISR */

/** MTU0 TGRA Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU0_TGIA0_ISR
#ifdef	EE_RX200_MTU0_TGIA0_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU0_TGIA0	EE_HWREG8(0x00087372U)
#endif	/* EE_RX200_MTU0_TGIA0_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU0_TGIA0	EE_HWREG8(0x0008720EU)
#define	EE_RX200_ICU_MTU0_TGIA0_MASK	0x00000004U
#endif	/* EE_RX200_MTU0_TGIA0_ISR */

/** MTU0 TGRB Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU0_TGIB0_ISR
#ifdef	EE_RX200_MTU0_TGIB0_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU0_TGIB0	EE_HWREG8(0x00087372U)
#endif	/* EE_RX200_MTU0_TGIB0_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU0_TGIB0	EE_HWREG8(0x0008720EU)
#define	EE_RX200_ICU_MTU0_TGIB0_MASK	0x00000008U
#endif	/* EE_RX200_MTU0_TGIB0_ISR */

/** MTU0 TGRC Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU0_TGIC0_ISR
#ifdef	EE_RX200_MTU0_TGIC0_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU0_TGIC0	EE_HWREG8(0x00087372U)
#endif	/* EE_RX200_MTU0_TGIC0_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU0_TGIC0	EE_HWREG8(0x0008720EU)
#define	EE_RX200_ICU_MTU0_TGIC0_MASK	0x00000010U
#endif	/* EE_RX200_MTU0_TGIC0_ISR */

/** MTU0 TGRD Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU0_TGID0_ISR
#ifdef	EE_RX200_MTU0_TGID0_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU0_TGID0	EE_HWREG8(0x00087372U)
#endif	/* EE_RX200_MTU0_TGID0_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU0_TGID0	EE_HWREG8(0x0008720EU)
#define	EE_RX200_ICU_MTU0_TGID0_MASK	0x00000020U
#endif	/* EE_RX200_MTU0_TGID0_ISR */

/** MTU0 Counter Overflow interrupt. */
#ifdef	EE_RX200_MTU0_TCIV0_ISR
#ifdef	EE_RX200_MTU0_TCIV0_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU0_TCIV0	EE_HWREG8(0x00087376U)
#endif	/* EE_RX200_MTU0_TCIV0_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU0_TCIV0	EE_HWREG8(0x0008720EU)
#define	EE_RX200_ICU_MTU0_TCIV0_MASK	0x00000040U
#endif	/* EE_RX200_MTU0_TCIV0_ISR */

/** MTU0 TGRE Compare Match interrupt. */
#ifdef	EE_RX200_MTU0_TGIE0_ISR
#ifdef	EE_RX200_MTU0_TGIE0_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU0_TGIE0	EE_HWREG8(0x00087376U)
#endif	/* EE_RX200_MTU0_TGIE0_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU0_TGIE0	EE_HWREG8(0x0008720EU)
#define	EE_RX200_ICU_MTU0_TGIE0_MASK	0x00000080U
#endif	/* EE_RX200_MTU0_TGIE0_ISR */

/** MTU0 TGRF Compare Match interrupt. */
#ifdef	EE_RX200_MTU0_TGIF0_ISR
#ifdef	EE_RX200_MTU0_TGIF0_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU0_TGIF0	EE_HWREG8(0x00087376U)
#endif	/* EE_RX200_MTU0_TGIF0_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU0_TGIF0	EE_HWREG8(0x0008720FU)
#define	EE_RX200_ICU_MTU0_TGIF0_MASK	0x00000001U
#endif	/* EE_RX200_MTU0_TGIF0_ISR */

/** MTU1 TGRA Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU1_TGIA1_ISR
#ifdef	EE_RX200_MTU1_TGIA1_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU1_TGIA1	EE_HWREG8(0x00087379U)
#endif	/* EE_RX200_MTU1_TGIA1_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU1_TGIA1	EE_HWREG8(0x0008720FU)
#define	EE_RX200_ICU_MTU1_TGIA1_MASK	0x00000002U
#endif	/* EE_RX200_MTU1_TGIA1_ISR */

/** MTU1 TGRB Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU1_TGIB1_ISR
#ifdef	EE_RX200_MTU1_TGIB1_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU1_TGIB1	EE_HWREG8(0x00087379U)
#endif	/* EE_RX200_MTU1_TGIB1_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU1_TGIB1	EE_HWREG8(0x0008720FU)
#define	EE_RX200_ICU_MTU1_TGIB1_MASK	0x00000004U
#endif	/* EE_RX200_MTU1_TGIB1_ISR */

/** MTU1 Counter Overflow interrupt. */
#ifdef	EE_RX200_MTU1_TCIV1_ISR
#ifdef	EE_RX200_MTU1_TCIV1_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU1_TCIV1	EE_HWREG8(0x0008737BU)
#endif	/* EE_RX200_MTU1_TCIV1_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU1_TCIV1	EE_HWREG8(0x0008720FU)
#define	EE_RX200_ICU_MTU1_TCIV1_MASK	0x00000008U
#endif	/* EE_RX200_MTU1_TCIV1_ISR */

/** MTU1 Counter Underflow interrupt. */
#ifdef	EE_RX200_MTU1_TCIU1_ISR
#ifdef	EE_RX200_MTU1_TCIU1_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU1_TCIU1	EE_HWREG8(0x0008737BU)
#endif	/* EE_RX200_MTU1_TCIU1_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU1_TCIU1	EE_HWREG8(0x0008720FU)
#define	EE_RX200_ICU_MTU1_TCIU1_MASK	0x00000010U
#endif	/* EE_RX200_MTU1_TCIU1_ISR */

/** MTU2 TGRA Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU2_TGIA2_ISR
#ifdef	EE_RX200_MTU2_TGIA2_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU2_TGIA2	EE_HWREG8(0x0008737DU)
#endif	/* EE_RX200_MTU2_TGIA2_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU2_TGIA2	EE_HWREG8(0x0008720FU)
#define	EE_RX200_ICU_MTU2_TGIA2_MASK	0x00000020U
#endif	/* EE_RX200_MTU2_TGIA2_ISR */

/** MTU2 TGRB Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU2_TGIB2_ISR
#ifdef	EE_RX200_MTU2_TGIB2_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU2_TGIB2	EE_HWREG8(0x0008737DU)
#endif	/* EE_RX200_MTU2_TGIB2_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU2_TGIB2	EE_HWREG8(0x0008720FU)
#define	EE_RX200_ICU_MTU2_TGIB2_MASK	0x00000040U
#endif	/* EE_RX200_MTU2_TGIB2_ISR */

/** MTU2 Counter Overflow interrupt. */
#ifdef	EE_RX200_MTU2_TCIV2_ISR
#ifdef	EE_RX200_MTU2_TCIV2_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU2_TCIV2	EE_HWREG8(0x0008737FU)
#endif	/* EE_RX200_MTU2_TCIV2_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU2_TCIV2	EE_HWREG8(0x0008720FU)
#define	EE_RX200_ICU_MTU2_TCIV2_MASK	0x00000080U
#endif	/* EE_RX200_MTU2_TCIV2_ISR */

/** MTU2 Counter Underflow interrupt. */
#ifdef	EE_RX200_MTU2_TCIU2_ISR
#ifdef	EE_RX200_MTU2_TCIU2_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU2_TCIU2	EE_HWREG8(0x00087381U)
#endif	/* EE_RX200_MTU2_TCIU2_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU2_TCIU2	EE_HWREG8(0x00087210U)
#define	EE_RX200_ICU_MTU2_TCIU2_MASK	0x00000001U
#endif	/* EE_RX200_MTU2_TCIU2_ISR */

/** MTU3 TGRA Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU3_TGIA3_ISR
#ifdef	EE_RX200_MTU3_TGIA3_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU3_TGIA3	EE_HWREG8(0x00087381U)
#endif	/* EE_RX200_MTU3_TGIA3_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU3_TGIA3	EE_HWREG8(0x00087210U)
#define	EE_RX200_ICU_MTU3_TGIA3_MASK	0x00000002U
#endif	/* EE_RX200_MTU3_TGIA3_ISR */

/** MTU3 TGRB Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU3_TGIB3_ISR
#ifdef	EE_RX200_MTU3_TGIB3_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU3_TGIB3	EE_HWREG8(0x00087381U)
#endif	/* EE_RX200_MTU3_TGIB3_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU3_TGIB3	EE_HWREG8(0x00087210U)
#define	EE_RX200_ICU_MTU3_TGIB3_MASK	0x00000004U
#endif	/* EE_RX200_MTU3_TGIB3_ISR */

/** MTU3 TGRC Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU3_TGIC3_ISR
#ifdef	EE_RX200_MTU3_TGIC3_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU3_TGIC3	EE_HWREG8(0x00087381U)
#endif	/* EE_RX200_MTU3_TGIC3_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU3_TGIC3	EE_HWREG8(0x00087210U)
#define	EE_RX200_ICU_MTU3_TGIC3_MASK	0x00000008U
#endif	/* EE_RX200_MTU3_TGIC3_ISR */

/** MTU3 TGRD Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU3_TGID3_ISR
#ifdef	EE_RX200_MTU3_TGID3_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU3_TGID3	EE_HWREG8(0x00087385U)
#endif	/* EE_RX200_MTU3_TGID3_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU3_TGID3	EE_HWREG8(0x00087210U)
#define	EE_RX200_ICU_MTU3_TGID3_MASK	0x00000010U
#endif	/* EE_RX200_MTU3_TGID3_ISR */

/** MTU3 Counter Overflow interrupt. */
#ifdef	EE_RX200_MTU3_TCIV3_ISR
#ifdef	EE_RX200_MTU3_TCIV3_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU3_TCIV3	EE_HWREG8(0x00087386U)
#endif	/* EE_RX200_MTU3_TCIV3_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU3_TCIV3	EE_HWREG8(0x00087210U)
#define	EE_RX200_ICU_MTU3_TCIV3_MASK	0x00000020U
#endif	/* EE_RX200_MTU3_TCIV3_ISR */

/** MTU4 TGRA Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU4_TGIA4_ISR
#ifdef	EE_RX200_MTU4_TGIA4_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU4_TGIA4	EE_HWREG8(0x00087386U)
#endif	/* EE_RX200_MTU4_TGIA4_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU4_TGIA4	EE_HWREG8(0x00087210U)
#define	EE_RX200_ICU_MTU4_TGIA4_MASK	0x00000040U
#endif	/* EE_RX200_MTU4_TGIA4_ISR */

/** MTU4 TGRB Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU4_TGIB4_ISR
#ifdef	EE_RX200_MTU4_TGIB4_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU4_TGIB4	EE_HWREG8(0x00087386U)
#endif	/* EE_RX200_MTU4_TGIB4_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU4_TGIB4	EE_HWREG8(0x00087210U)
#define	EE_RX200_ICU_MTU4_TGIB4_MASK	0x00000080U
#endif	/* EE_RX200_MTU4_TGIB4_ISR */

/** MTU4 TGRC Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU4_TGIC4_ISR
#ifdef	EE_RX200_MTU4_TGIC4_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU4_TGIC4	EE_HWREG8(0x00087386U)
#endif	/* EE_RX200_MTU4_TGIC4_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU4_TGIC4	EE_HWREG8(0x00087211U)
#define	EE_RX200_ICU_MTU4_TGIC4_MASK	0x00000001U
#endif	/* EE_RX200_MTU4_TGIC4_ISR */

/** MTU4 TGRD Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU4_TGID4_ISR
#ifdef	EE_RX200_MTU4_TGID4_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU4_TGID4	EE_HWREG8(0x00087386U)
#endif	/* EE_RX200_MTU3_TGID3_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU4_TGID4	EE_HWREG8(0x00087211U)
#define	EE_RX200_ICU_MTU4_TGID4_MASK	0x00000002U
#endif	/* EE_RX200_MTU4_TGID4_ISR */

/** MTU4 Counter Overflow interrupt. */
#ifdef	EE_RX200_MTU4_TCIV4_ISR
#ifdef	EE_RX200_MTU4_TCIV4_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU4_TCIV4	EE_HWREG8(0x0008738AU)
#endif	/* EE_RX200_MTU4_TCIV4_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU4_TCIV4	EE_HWREG8(0x00087211U)
#define	EE_RX200_ICU_MTU4_TCIV4_MASK	0x00000004U
#endif	/* EE_RX200_MTU4_TCIV4_ISR */

/** MTU5 TGRU Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU5_TGIU5_ISR
#ifdef	EE_RX200_MTU5_TGIU5_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU5_TGIU5	EE_HWREG8(0x0008738BU)
#endif	/* EE_RX200_MTU5_TGIU5_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU5_TGIU5	EE_HWREG8(0x00087211U)
#define	EE_RX200_ICU_MTU5_TGIU5_MASK	0x00000008U
#endif	/* EE_RX200_MTU5_TGIU5_ISR */

/** MTU5 TGRV Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU5_TGIV5_ISR
#ifdef	EE_RX200_MTU5_TGIV5_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU5_TGIV5	EE_HWREG8(0x0008738BU)
#endif	/* EE_RX200_MTU5_TGIV5_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU5_TGIV5	EE_HWREG8(0x00087211U)
#define	EE_RX200_ICU_MTU5_TGIV5_MASK	0x00000010U
#endif	/* EE_RX200_MTU5_TGIV5_ISR */

/** MTU5 TGRW Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_MTU5_TGIW5_ISR
#ifdef	EE_RX200_MTU5_TGIW5_ISR_PRI
#define	EE_RX200_ICU_IPR_MTU5_TGIW5	EE_HWREG8(0x0008738BU)
#endif	/* EE_RX200_MTU5_TGIW5_ISR_PRI */
#define	EE_RX200_ICU_IER_MTU5_TGIW5	EE_HWREG8(0x00087211U)
#define	EE_RX200_ICU_MTU5_TGIW5_MASK	0x00000020U
#endif	/* EE_RX200_MTU5_TGIW5_ISR */

/** TPU0 TGRA Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU0_TGI0A_ISR
#ifdef	EE_RX200_TPU0_TGI0A_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU0_TGI0A	EE_HWREG8(0x0008738EU)
#endif	/* EE_RX200_TPU0_TGI0A_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU0_TGI0A	EE_HWREG8(0x00087211U)
#define	EE_RX200_ICU_TPU0_TGI0A_MASK	0x00000040U
#endif	/* EE_RX200_TPU0_TGI0A_ISR */

/** TPU0 TGRB Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU0_TGI0B_ISR
#ifdef	EE_RX200_TPU0_TGI0B_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU0_TGI0B	EE_HWREG8(0x0008738EU)
#endif	/* EE_RX200_TPU0_TGI0B_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU0_TGI0B	EE_HWREG8(0x00087211U)
#define	EE_RX200_ICU_TPU0_TGI0B_MASK	0x00000080U
#endif	/* EE_RX200_TPU0_TGI0B_ISR */

/** TPU0 TGRC Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU0_TGI0C_ISR
#ifdef	EE_RX200_TPU0_TGI0C_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU0_TGI0C	EE_HWREG8(0x0008738EU)
#endif	/* EE_RX200_TPU0_TGI0C_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU0_TGI0C	EE_HWREG8(0x00087212U)
#define	EE_RX200_ICU_TPU0_TGI0C_MASK	0x00000001U
#endif	/* EE_RX200_TPU0_TGI0C_ISR */

/** TPU0 TGRD Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU0_TGI0D_ISR
#ifdef	EE_RX200_TPU0_TGI0D_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU0_TGI0D	EE_HWREG8(0x0008738EU)
#endif	/* EE_RX200_TPU0_TGI0D_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU0_TGI0D	EE_HWREG8(0x00087212U)
#define	EE_RX200_ICU_TPU0_TGI0D_MASK	0x00000002U
#endif	/* EE_RX200_TPU0_TGI0D_ISR */

/** TPU0 Counter Overflow interrupt. */
#ifdef	EE_RX200_TPU0_TCI0V_ISR
#ifdef	EE_RX200_TPU0_TCI0V_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU0_TCI0V	EE_HWREG8(0x00087392U)
#endif	/* EE_RX200_TPU0_TCI0V_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU0_TCI0V	EE_HWREG8(0x00087212U)
#define	EE_RX200_ICU_TPU0_TCI0V_MASK	0x00000004U
#endif	/* EE_RX200_TPU0_TCI0V_ISR */

/** TPU1 TGRA Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU1_TGI1A_ISR
#ifdef	EE_RX200_TPU1_TGI1A_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU1_TGI1A	EE_HWREG8(0x00087393U)
#endif	/* EE_RX200_TPU1_TGI1A_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU1_TGI1A	EE_HWREG8(0x00087212U)
#define	EE_RX200_ICU_TPU1_TGI1A_MASK	0x00000008U
#endif	/* EE_RX200_TPU1_TGI1A_ISR */

/** TPU1 TGRB Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU1_TGI1B_ISR
#ifdef	EE_RX200_TPU1_TGI1B_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU1_TGI1B	EE_HWREG8(0x00087393U)
#endif	/* EE_RX200_TPU1_TGI1B_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU1_TGI1B	EE_HWREG8(0x00087212U)
#define	EE_RX200_ICU_TPU1_TGI1B_MASK	0x00000010U
#endif	/* EE_RX200_TPU1_TGI1B_ISR */

/** TPU1 Counter Overflow interrupt. */
#ifdef	EE_RX200_TPU1_TCI1V_ISR
#ifdef	EE_RX200_TPU1_TCI1V_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU1_TCI1V	EE_HWREG8(0x00087395U)
#endif	/* EE_RX200_TPU1_TCI1V_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU1_TCI1V	EE_HWREG8(0x00087212U)
#define	EE_RX200_ICU_TPU1_TCI1V_MASK	0x00000020U
#endif	/* EE_RX200_TPU1_TCI1V_ISR */

/** TPU1 Counter Underflow interrupt. */
#ifdef	EE_RX200_TPU1_TCI1U_ISR
#ifdef	EE_RX200_TPU1_TCI1U_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU1_TCI1U	EE_HWREG8(0x00087395U)
#endif	/* EE_RX200_TPU1_TCI1U_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU1_TCI1U	EE_HWREG8(0x00087212U)
#define	EE_RX200_ICU_TPU1_TCI1U_MASK	0x00000040U
#endif	/* EE_RX200_TPU1_TCI1U_ISR */

/** TPU2 TGRA Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU2_TGI2A_ISR
#ifdef	EE_RX200_TPU2_TGI2A_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU2_TGI2A	EE_HWREG8(0x00087397U)
#endif	/* EE_RX200_TPU2_TGI2A_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU2_TGI2A	EE_HWREG8(0x00087212U)
#define	EE_RX200_ICU_TPU2_TGI2A_MASK	0x00000080U
#endif	/* EE_RX200_TPU2_TGI2A_ISR */

/** TPU2 TGRB Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU2_TGI2B_ISR
#ifdef	EE_RX200_TPU2_TGI2B_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU2_TGI2B	EE_HWREG8(0x00087397U)
#endif	/* EE_RX200_TPU2_TGI2B_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU2_TGI2B	EE_HWREG8(0x00087213U)
#define	EE_RX200_ICU_TPU2_TGI2B_MASK	0x00000001U
#endif	/* EE_RX200_TPU2_TGI2B_ISR */

/** TPU2 Counter Overflow interrupt. */
#ifdef	EE_RX200_TPU2_TCI2V_ISR
#ifdef	EE_RX200_TPU2_TCI2V_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU2_TCI2V	EE_HWREG8(0x00087399U)
#endif	/* EE_RX200_TPU2_TCI2V_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU2_TCI2V	EE_HWREG8(0x00087213U)
#define	EE_RX200_ICU_TPU2_TCI2V_MASK	0x00000002U
#endif	/* EE_RX200_TPU2_TCI2V_ISR */

/** TPU2 Counter Underflow interrupt. */
#ifdef	EE_RX200_TPU2_TCI2U_ISR
#ifdef	EE_RX200_TPU2_TCI2U_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU2_TCI2U	EE_HWREG8(0x00087399U)
#endif	/* EE_RX200_TPU2_TCI2U_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU2_TCI2U	EE_HWREG8(0x00087213U)
#define	EE_RX200_ICU_TPU2_TCI2U_MASK	0x00000004U
#endif	/* EE_RX200_TPU2_TCI2U_ISR */

/** TPU3 TGRA Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU3_TGI3A_ISR
#ifdef	EE_RX200_TPU3_TGI3A_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU3_TGI3A	EE_HWREG8(0x0008739BU)
#endif	/* EE_RX200_TPU3_TGI3A_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU3_TGI3A	EE_HWREG8(0x00087213U)
#define	EE_RX200_ICU_TPU3_TGI3A_MASK	0x00000008U
#endif	/* EE_RX200_TPU3_TGI3A_ISR */

/** TPU3 TGRB Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU3_TGI3B_ISR
#ifdef	EE_RX200_TPU3_TGI3B_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU3_TGI3B	EE_HWREG8(0x0008739BU)
#endif	/* EE_RX200_TPU3_TGI3B_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU3_TGI3B	EE_HWREG8(0x00087213U)
#define	EE_RX200_ICU_TPU3_TGI3B_MASK	0x00000010U
#endif	/* EE_RX200_TPU3_TGI3B_ISR */

/** TPU3 TGRC Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU3_TGI3C_ISR
#ifdef	EE_RX200_TPU3_TGI3C_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU3_TGI3C	EE_HWREG8(0x0008739BU)
#endif	/* EE_RX200_TPU3_TGI3C_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU3_TGI3C	EE_HWREG8(0x00087213U)
#define	EE_RX200_ICU_TPU3_TGI3C_MASK	0x00000020U
#endif	/* EE_RX200_TPU3_TGI3C_ISR */

/** TPU3 TGRD Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU3_TGI3D_ISR
#ifdef	EE_RX200_TPU3_TGI3D_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU3_TGI3D	EE_HWREG8(0x0008739BU)
#endif	/* EE_RX200_TPU3_TGI3D_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU3_TGI3D	EE_HWREG8(0x00087213U)
#define	EE_RX200_ICU_TPU3_TGI3D_MASK	0x00000040U
#endif	/* EE_RX200_TPU3_TGI3D_ISR */

/** TPU3 Counter Overflow interrupt. */
#ifdef	EE_RX200_TPU0_TCI0V_ISR
#ifdef	EE_RX200_TPU0_TCI0V_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU0_TCI0V	EE_HWREG8(0x0008739FU)
#endif	/* EE_RX200_TPU0_TCI0V_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU0_TCI0V	EE_HWREG8(0x00087213U)
#define	EE_RX200_ICU_TPU0_TCI0V_MASK	0x00000080U
#endif	/* EE_RX200_TPU0_TCI0V_ISR */

/** TPU4 TGRA Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU4_TGI4A_ISR
#ifdef	EE_RX200_TPU4_TGI4A_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU4_TGI4A	EE_HWREG8(0x000873A0U)
#endif	/* EE_RX200_TPU4_TGI4A_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU4_TGI4A	EE_HWREG8(0x00087214U)
#define	EE_RX200_ICU_TPU4_TGI4A_MASK	0x00000001U
#endif	/* EE_RX200_TPU4_TGI4A_ISR */

/** TPU4 TGRB Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU4_TGI4B_ISR
#ifdef	EE_RX200_TPU4_TGI4B_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU4_TGI4B	EE_HWREG8(0x000873A0U)
#endif	/* EE_RX200_TPU4_TGI4B_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU4_TGI4B	EE_HWREG8(0x00087214U)
#define	EE_RX200_ICU_TPU4_TGI4B_MASK	0x00000002U
#endif	/* EE_RX200_TPU4_TGI4B_ISR */

/** TPU4 Counter Overflow interrupt. */
#ifdef	EE_RX200_TPU4_TCI4V_ISR
#ifdef	EE_RX200_TPU4_TCI4V_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU4_TCI4V	EE_HWREG8(0x000873A2U)
#endif	/* EE_RX200_TPU4_TCI4V_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU4_TCI4V	EE_HWREG8(0x00087214U)
#define	EE_RX200_ICU_TPU4_TCI4V_MASK	0x00000004U
#endif	/* EE_RX200_TPU4_TCI4V_ISR */

/** TPU4 Counter Underflow interrupt. */
#ifdef	EE_RX200_TPU4_TCI4U_ISR
#ifdef	EE_RX200_TPU4_TCI4U_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU4_TCI4U	EE_HWREG8(0x000873A2U)
#endif	/* EE_RX200_TPU4_TCI4U_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU4_TCI4U	EE_HWREG8(0x00087214U)
#define	EE_RX200_ICU_TPU4_TCI4U_MASK	0x00000008U
#endif	/* EE_RX200_TPU4_TCI4U_ISR */

/** TPU5 TGRA Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU5_TGI5A_ISR
#ifdef	EE_RX200_TPU5_TGI5A_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU5_TGI5A	EE_HWREG8(0x000873A4U)
#endif	/* EE_RX200_TPU5_TGI5A_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU5_TGI5A	EE_HWREG8(0x00087214U)
#define	EE_RX200_ICU_TPU5_TGI5A_MASK	0x00000010U
#endif	/* EE_RX200_TPU5_TGI5A_ISR */

/** TPU5 TGRB Input Capture/Compare Match interrupt. */
#ifdef	EE_RX200_TPU5_TGI5B_ISR
#ifdef	EE_RX200_TPU5_TGI5B_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU5_TGI5B	EE_HWREG8(0x000873A4U)
#endif	/* EE_RX200_TPU5_TGI5B_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU5_TGI5B	EE_HWREG8(0x00087214U)
#define	EE_RX200_ICU_TPU5_TGI5B_MASK	0x00000020U
#endif	/* EE_RX200_TPU5_TGI5B_ISR */

/** TPU5 Counter Overflow interrupt. */
#ifdef	EE_RX200_TPU5_TCI5V_ISR
#ifdef	EE_RX200_TPU5_TCI5V_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU5_TCI5V	EE_HWREG8(0x000873A6U)
#endif	/* EE_RX200_TPU5_TCI5V_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU5_TCI5V	EE_HWREG8(0x00087214U)
#define	EE_RX200_ICU_TPU5_TCI5V_MASK	0x00000040U
#endif	/* EE_RX200_TPU5_TCI5V_ISR */

/** TPU5 Counter Underflow interrupt. */
#ifdef	EE_RX200_TPU5_TCI5U_ISR
#ifdef	EE_RX200_TPU5_TCI5U_ISR_PRI
#define	EE_RX200_ICU_IPR_TPU5_TCI5U	EE_HWREG8(0x000873A6U)
#endif	/* EE_RX200_TPU5_TCI5U_ISR_PRI */
#define	EE_RX200_ICU_IER_TPU5_TCI5U	EE_HWREG8(0x00087214U)
#define	EE_RX200_ICU_TPU5_TCI5U_MASK	0x00000080U
#endif	/* EE_RX200_TPU5_TCI5U_ISR */

/** Port Output 1 interrupt. */
#ifdef	EE_RX200_POE_OEI1_ISR
#ifdef	EE_RX200_POE_OEI1_ISR_PRI
#define	EE_RX200_ICU_IPR_POE_OEI1	EE_HWREG8(0x000873AAU)
#endif	/* EE_RX200_POE_OEI1_ISR_PRI */
#define	EE_RX200_ICU_IER_POE_OEI1	EE_HWREG8(0x00087215U)
#define	EE_RX200_ICU_POE_OEI1_MASK	0x00000004U
#endif	/* EE_RX200_POE_OEI1_ISR */

/** Port Output 2 interrupt. */
#ifdef	EE_RX200_POE_OEI2_ISR
#ifdef	EE_RX200_POE_OEI2_ISR_PRI
#define	EE_RX200_ICU_IPR_POE_OEI2	EE_HWREG8(0x000873ABU)
#endif	/* EE_RX200_POE_OEI2_ISR_PRI */
#define	EE_RX200_ICU_IER_POE_OEI2	EE_HWREG8(0x00087215U)
#define	EE_RX200_ICU_POE_OEI2_MASK	0x00000008U
#endif	/* EE_RX200_POE_OEI2_ISR */

/** TMR0 TCORA Compare Match interrupt. */
#ifdef	EE_RX200_TMR0_CMIA0_ISR
#ifdef	EE_RX200_TMR0_CMIA0_ISR_PRI
#define	EE_RX200_ICU_IPR_TMR0_CMIA0	EE_HWREG8(0x000873AEU)
#endif	/* EE_RX200_TMR0_CMIA0_ISR_PRI */
#define	EE_RX200_ICU_IER_TMR0_CMIA0	EE_HWREG8(0x00087215U)
#define	EE_RX200_ICU_TMR0_CMIA0_MASK	0x00000040U
#endif	/* EE_RX200_TMR0_CMIA0_ISR */

/** TMR0 TCORB Compare Match interrupt. */
#ifdef	EE_RX200_TMR0_CMIB0_ISR
#ifdef	EE_RX200_TMR0_CMIB0_ISR_PRI
#define	EE_RX200_ICU_IPR_TMR0_CMIB0	EE_HWREG8(0x000873AEU)
#endif	/* EE_RX200_TMR0_CMIB0_ISR_PRI */
#define	EE_RX200_ICU_IER_TMR0_CMIB0	EE_HWREG8(0x00087215U)
#define	EE_RX200_ICU_TMR0_CMIB0_MASK	0x00000080U
#endif	/* EE_RX200_TMR0_CMIB0_ISR */

/** TMR0 Counter Overflow interrupt. */
#ifdef	EE_RX200_TMR0_OVI0_ISR
#ifdef	EE_RX200_TMR0_OVI0_ISR_PRI
#define	EE_RX200_ICU_IPR_TMR0_OVI0	EE_HWREG8(0x000873AEU)
#endif	/* EE_RX200_TMR0_OVI0_ISR_PRI */
#define	EE_RX200_ICU_IER_TMR0_OVI0	EE_HWREG8(0x00087216U)
#define	EE_RX200_ICU_TMR0_OVI0_MASK	0x00000001U
#endif	/* EE_RX200_TMR0_OVI0_ISR */

/** TMR1 TCORA Compare Match interrupt. */
#ifdef	EE_RX200_TMR1_CMIA1_ISR
#ifdef	EE_RX200_TMR1_CMIA1_ISR_PRI
#define	EE_RX200_ICU_IPR_TMR1_CMIA1	EE_HWREG8(0x000873B1U)
#endif	/* EE_RX200_TMR1_CMIA1_ISR_PRI */
#define	EE_RX200_ICU_IER_TMR1_CMIA1	EE_HWREG8(0x00087216U)
#define	EE_RX200_ICU_TMR1_CMIA1_MASK	0x00000002U
#endif	/* EE_RX200_TMR1_CMIA0_ISR */

/** TMR1 TCORB Compare Match interrupt. */
#ifdef	EE_RX200_TMR1_CMIB1_ISR
#ifdef	EE_RX200_TMR1_CMIB1_ISR_PRI
#define	EE_RX200_ICU_IPR_TMR1_CMIB1	EE_HWREG8(0x000873B1U)
#endif	/* EE_RX200_TMR1_CMIB1_ISR_PRI */
#define	EE_RX200_ICU_IER_TMR1_CMIB1	EE_HWREG8(0x00087216U)
#define	EE_RX200_ICU_TMR1_CMIB1_MASK	0x00000004U
#endif	/* EE_RX200_TMR1_CMIB1_ISR */

/** TMR1 Counter Overflow interrupt. */
#ifdef	EE_RX200_TMR1_OVI1_ISR
#ifdef	EE_RX200_TMR1_OVI1_ISR_PRI
#define	EE_RX200_ICU_IPR_TMR1_OVI1	EE_HWREG8(0x000873B1U)
#endif	/* EE_RX200_TMR1_OVI1_ISR_PRI */
#define	EE_RX200_ICU_IER_TMR1_OVI1	EE_HWREG8(0x00087216U)
#define	EE_RX200_ICU_TMR1_OVI1_MASK	0x00000008U
#endif	/* EE_RX200_TMR1_OVI1_ISR */

/** TMR2 TCORA Compare Match interrupt. */
#ifdef	EE_RX200_TMR2_CMIA2_ISR
#ifdef	EE_RX200_TMR2_CMIA2_ISR_PRI
#define	EE_RX200_ICU_IPR_TMR2_CMIA2	EE_HWREG8(0x000873B4U)
#endif	/* EE_RX200_TMR2_CMIA2_ISR_PRI */
#define	EE_RX200_ICU_IER_TMR2_CMIA2	EE_HWREG8(0x00087216U)
#define	EE_RX200_ICU_TMR2_CMIA2_MASK	0x00000010U
#endif	/* EE_RX200_TMR2_CMIA0_ISR */

/** TMR2 TCORB Compare Match interrupt. */
#ifdef	EE_RX200_TMR2_CMIB2_ISR
#ifdef	EE_RX200_TMR2_CMIB2_ISR_PRI
#define	EE_RX200_ICU_IPR_TMR2_CMIB2	EE_HWREG8(0x000873B4U)
#endif	/* EE_RX200_TMR2_CMIB2_ISR_PRI */
#define	EE_RX200_ICU_IER_TMR2_CMIB2	EE_HWREG8(0x00087216U)
#define	EE_RX200_ICU_TMR2_CMIB2_MASK	0x00000020U
#endif	/* EE_RX200_TMR2_CMIB2_ISR */

/** TMR2 Counter Overflow interrupt. */
#ifdef	EE_RX200_TMR2_OVI2_ISR
#ifdef	EE_RX200_TMR2_OVI2_ISR_PRI
#define	EE_RX200_ICU_IPR_TMR2_OVI2	EE_HWREG8(0x000873B4U)
#endif	/* EE_RX200_TMR2_OVI2_ISR_PRI */
#define	EE_RX200_ICU_IER_TMR2_OVI2	EE_HWREG8(0x00087216U)
#define	EE_RX200_ICU_TMR2_OVI2_MASK	0x00000040U
#endif	/* EE_RX200_TMR2_OVI2_ISR */

/** TMR3 TCORA Compare Match interrupt. */
#ifdef	EE_RX200_TMR3_CMIA3_ISR
#ifdef	EE_RX200_TMR3_CMIA3_ISR_PRI
#define	EE_RX200_ICU_IPR_TMR3_CMIA3	EE_HWREG8(0x000873B7U)
#endif	/* EE_RX200_TMR3_CMIA3_ISR_PRI */
#define	EE_RX200_ICU_IER_TMR3_CMIA3	EE_HWREG8(0x00087216U)
#define	EE_RX200_ICU_TMR3_CMIA3_MASK	0x00000080U
#endif	/* EE_RX200_TMR3_CMIA0_ISR */

/** TMR3 TCORB Compare Match interrupt. */
#ifdef	EE_RX200_TMR3_CMIB3_ISR
#ifdef	EE_RX200_TMR3_CMIB3_ISR_PRI
#define	EE_RX200_ICU_IPR_TMR3_CMIB3	EE_HWREG8(0x000873B7U)
#endif	/* EE_RX200_TMR3_CMIB3_ISR_PRI */
#define	EE_RX200_ICU_IER_TMR3_CMIB3	EE_HWREG8(0x00087217U)
#define	EE_RX200_ICU_TMR3_CMIB3_MASK	0x00000001U
#endif	/* EE_RX200_TMR3_CMIB3_ISR */

/** TMR3 Counter Overflow interrupt. */
#ifdef	EE_RX200_TMR3_OVI3_ISR
#ifdef	EE_RX200_TMR3_OVI3_ISR_PRI
#define	EE_RX200_ICU_IPR_TMR3_OVI3	EE_HWREG8(0x000873B7U)
#endif	/* EE_RX200_TMR3_OVI3_ISR_PRI */
#define	EE_RX200_ICU_IER_TMR3_OVI3	EE_HWREG8(0x00087217U)
#define	EE_RX200_ICU_TMR3_OVI3_MASK	0x00000002U
#endif	/* EE_RX200_TMR3_OVI3_ISR */

/** SCI2 RX Error interrupt. */
#ifdef	EE_RX200_SCI2_ERI2_ISR
#ifdef	EE_RX200_SCI2_ERI2_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI2_ERI2	EE_HWREG8(0x000873BAU)
#endif	/* EE_RX200_SCI2_ERI2_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI2_ERI2	EE_HWREG8(0x00087217U)
#define	EE_RX200_ICU_SCI2_ERI2_MASK	0x00000003U
#endif	/* EE_RX200_SCI2_ERI2_ISR */

/** SCI2 RX interrupt. */
#ifdef	EE_RX200_SCI2_RXI2_ISR
#ifdef	EE_RX200_SCI2_RXI2_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI2_RXI2	EE_HWREG8(0x000873BAU)
#endif	/* EE_RX200_SCI2_RXI2_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI2_RXI2	EE_HWREG8(0x00087217U)
#define	EE_RX200_ICU_SCI2_RXI2_MASK	0x00000008U
#endif	/* EE_RX200_SCI2_RXI2_ISR */

/** SCI2 TX Empty interrupt. */
#ifdef	EE_RX200_SCI2_TXI2_ISR
#ifdef	EE_RX200_SCI2_TXI2_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI2_TXI2	EE_HWREG8(0x000873BAU)
#endif	/* EE_RX200_SCI2_TXI2_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI2_TXI2	EE_HWREG8(0x00087217U)
#define	EE_RX200_ICU_SCI2_TXI2_MASK	0x00000010U
#endif	/* EE_RX200_SCI2_TXI2_ISR */

/** SCI2 TX End interrupt. */
#ifdef	EE_RX200_SCI2_TEI2_ISR
#ifdef	EE_RX200_SCI2_TEI2_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI2_TEI2	EE_HWREG8(0x000873BAU)
#endif	/* EE_RX200_SCI2_TEI2_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI2_TEI2	EE_HWREG8(0x00087217U)
#define	EE_RX200_ICU_SCI2_TEI2_MASK	0x00000020U
#endif	/* EE_RX200_SCI2_TEI2_ISR */

/** SCI3 RX Error interrupt. */
#ifdef	EE_RX200_SCI3_ERI3_ISR
#ifdef	EE_RX200_SCI3_ERI3_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI3_ERI3	EE_HWREG8(0x000873BEU)
#endif	/* EE_RX200_SCI3_ERI3_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI3_ERI3	EE_HWREG8(0x00087217U)
#define	EE_RX200_ICU_SCI3_ERI3_MASK	0x00000040U
#endif	/* EE_RX200_SCI3_ERI3_ISR */

/** SCI3 RX interrupt. */
#ifdef	EE_RX200_SCI3_RXI3_ISR
#ifdef	EE_RX200_SCI3_RXI3_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI3_RXI3	EE_HWREG8(0x000873BEU)
#endif	/* EE_RX200_SCI3_RXI3_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI3_RXI3	EE_HWREG8(0x00087217U)
#define	EE_RX200_ICU_SCI3_RXI3_MASK	0x00000080U
#endif	/* EE_RX200_SCI3_RXI3_ISR */

/** SCI3 TX Empty interrupt. */
#ifdef	EE_RX200_SCI3_TXI3_ISR
#ifdef	EE_RX200_SCI3_TXI3_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI3_TXI3	EE_HWREG8(0x000873BEU)
#endif	/* EE_RX200_SCI3_TXI3_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI3_TXI3	EE_HWREG8(0x00087218U)
#define	EE_RX200_ICU_SCI3_TXI3_MASK	0x00000001U
#endif	/* EE_RX200_SCI3_TXI3_ISR */

/** SCI3 TX End interrupt. */
#ifdef	EE_RX200_SCI3_TEI3_ISR
#ifdef	EE_RX200_SCI3_TEI3_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI3_TEI3	EE_HWREG8(0x000873BEU)
#endif	/* EE_RX200_SCI3_TEI3_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI3_TEI3	EE_HWREG8(0x00087218U)
#define	EE_RX200_ICU_SCI3_TEI3_MASK	0x00000002U
#endif	/* EE_RX200_SCI3_TEI3_ISR */

/** SCI4 RX Error interrupt. */
#ifdef	EE_RX200_SCI4_ERI4_ISR
#ifdef	EE_RX200_SCI4_ERI4_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI4_ERI4	EE_HWREG8(0x000873C2U)
#endif	/* EE_RX200_SCI4_ERI4_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI4_ERI4	EE_HWREG8(0x00087218U)
#define	EE_RX200_ICU_SCI4_ERI4_MASK	0x00000004U
#endif	/* EE_RX200_SCI4_ERI4_ISR */

/** SCI4 RX interrupt. */
#ifdef	EE_RX200_SCI4_RXI4_ISR
#ifdef	EE_RX200_SCI4_RXI4_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI4_RXI4	EE_HWREG8(0x000873C2U)
#endif	/* EE_RX200_SCI4_RXI4_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI4_RXI4	EE_HWREG8(0x00087218U)
#define	EE_RX200_ICU_SCI4_RXI4_MASK	0x00000008U
#endif	/* EE_RX200_SCI4_RXI4_ISR */

/** SCI4 TX Empty interrupt. */
#ifdef	EE_RX200_SCI4_TXI4_ISR
#ifdef	EE_RX200_SCI4_TXI4_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI4_TXI4	EE_HWREG8(0x000873C2U)
#endif	/* EE_RX200_SCI4_TXI4_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI4_TXI4	EE_HWREG8(0x00087218U)
#define	EE_RX200_ICU_SCI4_TXI4_MASK	0x00000010U
#endif	/* EE_RX200_SCI4_TXI4_ISR */

/** SCI4 TX End interrupt. */
#ifdef	EE_RX200_SCI4_TEI4_ISR
#ifdef	EE_RX200_SCI4_TEI4_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI4_TEI4	EE_HWREG8(0x000873C2U)
#endif	/* EE_RX200_SCI4_TEI4_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI4_TEI4	EE_HWREG8(0x00087218U)
#define	EE_RX200_ICU_SCI4_TEI4_MASK	0x00000020U
#endif	/* EE_RX200_SCI4_TEI4_ISR */

/** DMA Channel 0 End interrupt. */
#ifdef	EE_RX200_DMAC_DMAC0I_ISR
#ifdef	EE_RX200_DMAC_DMAC0I_ISR_PRI
#define	EE_RX200_ICU_IPR_DMAC_DMAC0I	EE_HWREG8(0x000873C6U)
#endif	/* EE_RX200_DMAC_DMAC0I_ISR_PRI */
#define	EE_RX200_ICU_IER_DMAC_DMAC0I	EE_HWREG8(0x00087218U)
#define	EE_RX200_ICU_DMAC_DMAC0I_MASK	0x00000040U
#endif	/* EE_RX200_DMAC_DMAC0I_ISR */

/** DMA Channel 1 End interrupt. */
#ifdef	EE_RX200_DMAC_DMAC1I_ISR
#ifdef	EE_RX200_DMAC_DMAC1I_ISR_PRI
#define	EE_RX200_ICU_IPR_DMAC_DMAC1I	EE_HWREG8(0x000873C7U)
#endif	/* EE_RX200_DMAC_DMAC1I_ISR_PRI */
#define	EE_RX200_ICU_IER_DMAC_DMAC1I	EE_HWREG8(0x00087218U)
#define	EE_RX200_ICU_DMAC_DMAC1I_MASK	0x00000080U
#endif	/* EE_RX200_DMAC_DMAC1I_ISR */

/** DMA Channel 2 End interrupt. */
#ifdef	EE_RX200_DMAC_DMAC2I_ISR
#ifdef	EE_RX200_DMAC_DMAC2I_ISR_PRI
#define	EE_RX200_ICU_IPR_DMAC_DMAC2I	EE_HWREG8(0x000873C8U)
#endif	/* EE_RX200_DMAC_DMAC2I_ISR_PRI */
#define	EE_RX200_ICU_IER_DMAC_DMAC2I	EE_HWREG8(0x00087219U)
#define	EE_RX200_ICU_DMAC_DMAC2I_MASK	0x00000010U
#endif	/* EE_RX200_DMAC_DMAC2I_ISR */

/** DMA Channel 3 End interrupt. */
#ifdef	EE_RX200_DMAC_DMAC3I_ISR
#ifdef	EE_RX200_DMAC_DMAC3I_ISR_PRI
#define	EE_RX200_ICU_IPR_DMAC_DMAC3I	EE_HWREG8(0x000873C9U)
#endif	/* EE_RX200_DMAC_DMAC3I_ISR_PRI */
#define	EE_RX200_ICU_IER_DMAC_DMAC3I	EE_HWREG8(0x00087219U)
#define	EE_RX200_ICU_DMAC_DMAC3I_MASK	0x00000020U
#endif	/* EE_RX200_DMAC_DMAC3I_ISR */

/** SCI7 RX Error interrupt. */
#ifdef	EE_RX200_SCI7_ERI7_ISR
#ifdef	EE_RX200_SCI7_ERI7_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI7_ERI7	EE_HWREG8(0x000873CEU)
#endif	/* EE_RX200_SCI7_ERI7_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI7_ERI7	EE_HWREG8(0x00087219U)
#define	EE_RX200_ICU_SCI7_ERI7_MASK	0x00000040U
#endif	/* EE_RX200_SCI7_ERI7_ISR */

/** SCI7 RX interrupt. */
#ifdef	EE_RX200_SCI7_RXI7_ISR
#ifdef	EE_RX200_SCI7_RXI7_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI7_RXI7	EE_HWREG8(0x000873CEU)
#endif	/* EE_RX200_SCI7_RXI7_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI7_RXI7	EE_HWREG8(0x00087219U)
#define	EE_RX200_ICU_SCI7_RXI7_MASK	0x00000080U
#endif	/* EE_RX200_SCI7_RXI7_ISR */

/** SCI7 TX Empty interrupt. */
#ifdef	EE_RX200_SCI7_TXI7_ISR
#ifdef	EE_RX200_SCI7_TXI7_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI7_TXI7	EE_HWREG8(0x000873CEU)
#endif	/* EE_RX200_SCI7_TXI7_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI7_TXI7	EE_HWREG8(0x0008721AU)
#define	EE_RX200_ICU_SCI7_TXI7_MASK	0x00000001U
#endif	/* EE_RX200_SCI7_TXI7_ISR */

/** SCI7 TX End interrupt. */
#ifdef	EE_RX200_SCI7_TEI7_ISR
#ifdef	EE_RX200_SCI7_TEI7_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI7_TEI7	EE_HWREG8(0x000873CEU)
#endif	/* EE_RX200_SCI7_TEI7_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI7_TEI7	EE_HWREG8(0x0008721AU)
#define	EE_RX200_ICU_SCI7_TEI7_MASK	0x00000002U
#endif	/* EE_RX200_SCI7_TEI7_ISR */

/** SCI10 RX Error interrupt. */
#ifdef	EE_RX200_SCI10_ERI10_ISR
#ifdef	EE_RX200_SCI10_ERI10_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI10_ERI10	EE_HWREG8(0x000873D2U)
#endif	/* EE_RX200_SCI10_ERI10_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI10_ERI10	EE_HWREG8(0x0008721AU)
#define	EE_RX200_ICU_SCI10_ERI10_MASK	0x00000004U
#endif	/* EE_RX200_SCI10_ERI10_ISR */

/** SCI10 RX interrupt. */
#ifdef	EE_RX200_SCI10_RXI10_ISR
#ifdef	EE_RX200_SCI10_RXI10_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI10_RXI10	EE_HWREG8(0x000873D2U)
#endif	/* EE_RX200_SCI10_RXI10_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI10_RXI10	EE_HWREG8(0x0008721AU)
#define	EE_RX200_ICU_SCI10_RXI10_MASK	0x00000008U
#endif	/* EE_RX200_SCI10_RXI10_ISR */

/** SCI10 TX Empty interrupt. */
#ifdef	EE_RX200_SCI10_TXI10_ISR
#ifdef	EE_RX200_SCI10_TXI10_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI10_TXI10	EE_HWREG8(0x000873D2U)
#endif	/* EE_RX200_SCI10_TXI10_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI10_TXI10	EE_HWREG8(0x0008721BU)
#define	EE_RX200_ICU_SCI10_TXI10_MASK	0x00000010U
#endif	/* EE_RX200_SCI10_TXI10_ISR */

/** SCI10 TX End interrupt. */
#ifdef	EE_RX200_SCI10_TEI10_ISR
#ifdef	EE_RX200_SCI10_TEI10_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI10_TEI10	EE_HWREG8(0x000873D2U)
#endif	/* EE_RX200_SCI10_TEI10_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI10_TEI10	EE_HWREG8(0x0008721BU)
#define	EE_RX200_ICU_SCI10_TEI10_MASK	0x00000020U
#endif	/* EE_RX200_SCI10_TEI10_ISR */

/** SCI0 RX Error interrupt. */
#ifdef	EE_RX200_SCI0_ERI0_ISR
#ifdef	EE_RX200_SCI0_ERI0_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI0_ERI0	EE_HWREG8(0x000873D6U)
#endif	/* EE_RX200_SCI0_ERI0_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI0_ERI0	EE_HWREG8(0x0008721AU)
#define	EE_RX200_ICU_SCI0_ERI0_MASK	0x00000040U
#endif	/* EE_RX200_SCI0_ERI0_ISR */

/** SCI0 RX interrupt. */
#ifdef	EE_RX200_SCI0_RXI0_ISR
#ifdef	EE_RX200_SCI0_RXI0_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI0_RXI0	EE_HWREG8(0x000873D6U)
#endif	/* EE_RX200_SCI0_RXI0_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI0_RXI0	EE_HWREG8(0x0008721AU)
#define	EE_RX200_ICU_SCI0_RXI0_MASK	0x00000080U
#endif	/* EE_RX200_SCI0_RXI0_ISR */

/** SCI0 TX Empty interrupt. */
#ifdef	EE_RX200_SCI0_TXI0_ISR
#ifdef	EE_RX200_SCI0_TXI0_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI0_TXI0	EE_HWREG8(0x000873D6U)
#endif	/* EE_RX200_SCI0_TXI0_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI0_TXI0	EE_HWREG8(0x0008721BU)
#define	EE_RX200_ICU_SCI0_TXI0_MASK	0x00000001U
#endif	/* EE_RX200_SCI0_TXI0_ISR */

/** SCI0 TX End interrupt. */
#ifdef	EE_RX200_SCI0_TEI0_ISR
#ifdef	EE_RX200_SCI0_TEI0_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI0_TEI0	EE_HWREG8(0x000873D6U)
#endif	/* EE_RX200_SCI0_TEI0_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI0_TEI0	EE_HWREG8(0x0008721BU)
#define	EE_RX200_ICU_SCI0_TEI0_MASK	0x00000002U
#endif	/* EE_RX200_SCI0_TEI0_ISR */

/** SCI1 RX Error interrupt. */
#ifdef	EE_RX200_SCI1_ERI1_ISR
#ifdef	EE_RX200_SCI1_ERI1_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI1_ERI1	EE_HWREG8(0x000873DAU)
#endif	/* EE_RX200_SCI1_ERI1_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI1_ERI1	EE_HWREG8(0x0008721BU)
#define	EE_RX200_ICU_SCI1_ERI1_MASK	0x00000004U
#endif	/* EE_RX200_SCI1_ERI1_ISR */

/** SCI1 RX interrupt. */
#ifdef	EE_RX200_SCI1_RXI1_ISR
#ifdef	EE_RX200_SCI1_RXI1_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI1_RXI1	EE_HWREG8(0x000873DAU)
#endif	/* EE_RX200_SCI1_RXI1_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI1_RXI1	EE_HWREG8(0x0008721BU)
#define	EE_RX200_ICU_SCI1_RXI1_MASK	0x00000008U
#endif	/* EE_RX200_SCI1_RXI1_ISR */

/** SCI1 TX Empty interrupt. */
#ifdef	EE_RX200_SCI1_TXI1_ISR
#ifdef	EE_RX200_SCI1_TXI1_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI1_TXI1	EE_HWREG8(0x000873DAU)
#endif	/* EE_RX200_SCI1_TXI1_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI1_TXI1	EE_HWREG8(0x0008721BU)
#define	EE_RX200_ICU_SCI1_TXI1_MASK	0x00000010U
#endif	/* EE_RX200_SCI1_TXI1_ISR */

/** SCI1 TX End interrupt. */
#ifdef	EE_RX200_SCI1_TEI1_ISR
#ifdef	EE_RX200_SCI1_TEI1_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI1_TEI1	EE_HWREG8(0x000873DAU)
#endif	/* EE_RX200_SCI1_TEI1_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI1_TEI1	EE_HWREG8(0x0008721BU)
#define	EE_RX200_ICU_SCI1_TEI1_MASK	0x00000020U
#endif	/* EE_RX200_SCI1_TEI1_ISR */

/** SCI5 RX Error interrupt. */
#ifdef	EE_RX200_SCI5_ERI5_ISR
#ifdef	EE_RX200_SCI5_ERI5_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI5_ERI5	EE_HWREG8(0x000873DEU)
#endif	/* EE_RX200_SCI5_ERI5_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI5_ERI5	EE_HWREG8(0x0008721BU)
#define	EE_RX200_ICU_SCI5_ERI5_MASK	0x00000040U
#endif	/* EE_RX200_SCI5_ERI5_ISR */

/** SCI5 RX interrupt. */
#ifdef	EE_RX200_SCI5_RXI5_ISR
#ifdef	EE_RX200_SCI5_RXI5_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI5_RXI5	EE_HWREG8(0x000873DEU)
#endif	/* EE_RX200_SCI5_RXI5_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI5_RXI5	EE_HWREG8(0x0008721BU)
#define	EE_RX200_ICU_SCI5_RXI5_MASK	0x00000080U
#endif	/* EE_RX200_SCI5_RXI5_ISR */

/** SCI5 TX Empty interrupt. */
#ifdef	EE_RX200_SCI5_TXI5_ISR
#ifdef	EE_RX200_SCI5_TXI5_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI5_TXI5	EE_HWREG8(0x000873DEU)
#endif	/* EE_RX200_SCI5_TXI5_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI5_TXI5	EE_HWREG8(0x0008721CU)
#define	EE_RX200_ICU_SCI5_TXI5_MASK	0x00000001U
#endif	/* EE_RX200_SCI5_TXI5_ISR */

/** SCI5 TX End interrupt. */
#ifdef	EE_RX200_SCI5_TEI5_ISR
#ifdef	EE_RX200_SCI5_TEI5_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI5_TEI5	EE_HWREG8(0x000873DEU)
#endif	/* EE_RX200_SCI5_TEI5_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI5_TEI5	EE_HWREG8(0x0008721CU)
#define	EE_RX200_ICU_SCI5_TEI5_MASK	0x00000002U
#endif	/* EE_RX200_SCI5_TEI5_ISR */

/** SCI6 RX Error interrupt. */
#ifdef	EE_RX200_SCI6_ERI6_ISR
#ifdef	EE_RX200_SCI6_ERI6_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI6_ERI6	EE_HWREG8(0x000873E2U)
#endif	/* EE_RX200_SCI6_ERI6_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI6_ERI6	EE_HWREG8(0x0008721CU)
#define	EE_RX200_ICU_SCI6_ERI6_MASK	0x00000004U
#endif	/* EE_RX200_SCI6_ERI6_ISR */

/** SCI6 RX interrupt. */
#ifdef	EE_RX200_SCI6_RXI6_ISR
#ifdef	EE_RX200_SCI6_RXI6_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI6_RXI6	EE_HWREG8(0x000873E2U)
#endif	/* EE_RX200_SCI6_RXI6_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI6_RXI6	EE_HWREG8(0x0008721CU)
#define	EE_RX200_ICU_SCI6_RXI6_MASK	0x00000008U
#endif	/* EE_RX200_SCI6_RXI6_ISR */

/** SCI6 TX Empty interrupt. */
#ifdef	EE_RX200_SCI6_TXI6_ISR
#ifdef	EE_RX200_SCI6_TXI6_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI6_TXI6	EE_HWREG8(0x000873E2U)
#endif	/* EE_RX200_SCI6_TXI6_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI6_TXI6	EE_HWREG8(0x0008721CU)
#define	EE_RX200_ICU_SCI6_TXI6_MASK	0x00000010U
#endif	/* EE_RX200_SCI6_TXI6_ISR */

/** SCI6 TX End interrupt. */
#ifdef	EE_RX200_SCI6_TEI6_ISR
#ifdef	EE_RX200_SCI6_TEI6_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI6_TEI6	EE_HWREG8(0x000873E2U)
#endif	/* EE_RX200_SCI6_TEI6_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI6_TEI6	EE_HWREG8(0x0008721CU)
#define	EE_RX200_ICU_SCI6_TEI6_MASK	0x00000020U
#endif	/* EE_RX200_SCI6_TEI6_ISR */

/** SCI8 RX Error interrupt. */
#ifdef	EE_RX200_SCI8_ERI8_ISR
#ifdef	EE_RX200_SCI8_ERI8_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI8_ERI8	EE_HWREG8(0x000873E6U)
#endif	/* EE_RX200_SCI8_ERI8_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI8_ERI8	EE_HWREG8(0x0008721CU)
#define	EE_RX200_ICU_SCI8_ERI8_MASK	0x00000040U
#endif	/* EE_RX200_SCI8_ERI8_ISR */

/** SCI8 RX interrupt. */
#ifdef	EE_RX200_SCI8_RXI8_ISR
#ifdef	EE_RX200_SCI8_RXI8_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI8_RXI8	EE_HWREG8(0x000873E6U)
#endif	/* EE_RX200_SCI8_RXI8_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI8_RXI8	EE_HWREG8(0x0008721CU)
#define	EE_RX200_ICU_SCI8_RXI8_MASK	0x00000080U
#endif	/* EE_RX200_SCI8_RXI8_ISR */

/** SCI8 TX Empty interrupt. */
#ifdef	EE_RX200_SCI8_TXI8_ISR
#ifdef	EE_RX200_SCI8_TXI8_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI8_TXI8	EE_HWREG8(0x000873E6U)
#endif	/* EE_RX200_SCI8_TXI8_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI8_TXI8	EE_HWREG8(0x0008721DU)
#define	EE_RX200_ICU_SCI8_TXI8_MASK	0x00000001U
#endif	/* EE_RX200_SCI8_TXI8_ISR */

/** SCI8 TX End interrupt. */
#ifdef	EE_RX200_SCI8_TEI8_ISR
#ifdef	EE_RX200_SCI8_TEI8_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI8_TEI8	EE_HWREG8(0x000873E6U)
#endif	/* EE_RX200_SCI8_TEI8_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI8_TEI8	EE_HWREG8(0x0008721DU)
#define	EE_RX200_ICU_SCI8_TEI8_MASK	0x00000002U
#endif	/* EE_RX200_SCI8_TEI8_ISR */

/** SCI9 RX Error interrupt. */
#ifdef	EE_RX200_SCI9_ERI9_ISR
#ifdef	EE_RX200_SCI9_ERI9_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI9_ERI9	EE_HWREG8(0x000873EAU)
#endif	/* EE_RX200_SCI9_ERI9_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI9_ERI9	EE_HWREG8(0x0008721DU)
#define	EE_RX200_ICU_SCI9_ERI9_MASK	0x00000004U
#endif	/* EE_RX200_SCI9_ERI9_ISR */

/** SCI9 RX interrupt. */
#ifdef	EE_RX200_SCI9_RXI9_ISR
#ifdef	EE_RX200_SCI9_RXI9_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI9_RXI9	EE_HWREG8(0x000873EAU)
#endif	/* EE_RX200_SCI9_RXI9_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI9_RXI9	EE_HWREG8(0x0008721DU)
#define	EE_RX200_ICU_SCI9_RXI9_MASK	0x00000008U
#endif	/* EE_RX200_SCI9_RXI9_ISR */

/** SCI9 TX Empty interrupt. */
#ifdef	EE_RX200_SCI9_TXI9_ISR
#ifdef	EE_RX200_SCI9_TXI9_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI9_TXI9	EE_HWREG8(0x000873EAU)
#endif	/* EE_RX200_SCI9_TXI9_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI9_TXI9	EE_HWREG8(0x0008721DU)
#define	EE_RX200_ICU_SCI9_TXI9_MASK	0x00000010U
#endif	/* EE_RX200_SCI9_TXI9_ISR */

/** SCI9 TX End interrupt. */
#ifdef	EE_RX200_SCI9_TEI9_ISR
#ifdef	EE_RX200_SCI9_TEI9_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI9_TEI9	EE_HWREG8(0x000873EAU)
#endif	/* EE_RX200_SCI9_TEI9_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI9_TEI9	EE_HWREG8(0x0008721DU)
#define	EE_RX200_ICU_SCI9_TEI9_MASK	0x00000020U
#endif	/* EE_RX200_SCI9_TEI9_ISR */

/** SCI12 RX Error interrupt. */
#ifdef	EE_RX200_SCI12_ERI12_ISR
#ifdef	EE_RX200_SCI12_ERI12_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI12_ERI12	EE_HWREG8(0x000873EEU)
#endif	/* EE_RX200_SCI12_ERI12_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI12_ERI12	EE_HWREG8(0x0008721DU)
#define	EE_RX200_ICU_SCI12_ERI12_MASK	0x00000040U
#endif	/* EE_RX200_SCI12_ERI12_ISR */

/** SCI12 RX interrupt. */
#ifdef	EE_RX200_SCI12_RXI12_ISR
#ifdef	EE_RX200_SCI12_RXI12_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI12_RXI12	EE_HWREG8(0x000873EEU)
#endif	/* EE_RX200_SCI12_RXI12_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI12_RXI12	EE_HWREG8(0x0008721DU)
#define	EE_RX200_ICU_SCI12_RXI12_MASK	0x00000080U
#endif	/* EE_RX200_SCI12_RXI12_ISR */

/** SCI12 TX Empty interrupt. */
#ifdef	EE_RX200_SCI12_TXI12_ISR
#ifdef	EE_RX200_SCI12_TXI12_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI12_TXI12	EE_HWREG8(0x000873EEU)
#endif	/* EE_RX200_SCI12_TXI12_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI12_TXI12	EE_HWREG8(0x0008721EU)
#define	EE_RX200_ICU_SCI12_TXI12_MASK	0x00000001U
#endif	/* EE_RX200_SCI12_TXI12_ISR */

/** SCI12 TX End interrupt. */
#ifdef	EE_RX200_SCI12_TEI12_ISR
#ifdef	EE_RX200_SCI12_TEI12_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI12_TEI12	EE_HWREG8(0x000873EEU)
#endif	/* EE_RX200_SCI12_TEI12_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI12_TEI12	EE_HWREG8(0x0008721EU)
#define	EE_RX200_ICU_SCI12_TEI12_MASK	0x00000002U
#endif	/* EE_RX200_SCI12_TEI12_ISR */

/** SCI12 Extended Serial Mode Control 0 interrupt. */
#ifdef	EE_RX200_SCI12_SCIX0_ISR
#ifdef	EE_RX200_SCI12_SCIX0_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI12_SCIX0	EE_HWREG8(0x000873F2U)
#endif	/* EE_RX200_SCI12_SCIX0_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI12_SCIX0	EE_HWREG8(0x0008721EU)
#define	EE_RX200_ICU_SCI12_SCIX0_MASK	0x00000004U
#endif	/* EE_RX200_SCI12_SCIX0_ISR */

/** SCI12 Extended Serial Mode Control 1 interrupt. */
#ifdef	EE_RX200_SCI12_SCIX1_ISR
#ifdef	EE_RX200_SCI12_SCIX1_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI12_SCIX1	EE_HWREG8(0x000873F3U)
#endif	/* EE_RX200_SCI12_SCIX1_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI12_SCIX1	EE_HWREG8(0x0008721EU)
#define	EE_RX200_ICU_SCI12_SCIX1_MASK	0x00000008U
#endif	/* EE_RX200_SCI12_SCIX1_ISR */

/** SCI12 Extended Serial Mode Control 2 interrupt. */
#ifdef	EE_RX200_SCI12_SCIX2_ISR
#ifdef	EE_RX200_SCI12_SCIX2_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI12_SCIX2	EE_HWREG8(0x000873F4U)
#endif	/* EE_RX200_SCI12_SCIX2_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI12_SCIX2	EE_HWREG8(0x0008721EU)
#define	EE_RX200_ICU_SCI12_SCIX2_MASK	0x00000010U
#endif	/* EE_RX200_SCI12_SCIX2_ISR */

/** SCI12 Extended Serial Mode Control 3 interrupt. */
#ifdef	EE_RX200_SCI12_SCIX3_ISR
#ifdef	EE_RX200_SCI12_SCIX3_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI12_SCIX3	EE_HWREG8(0x000873F5U)
#endif	/* EE_RX200_SCI12_SCIX3_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI12_SCIX3	EE_HWREG8(0x0008721EU)
#define	EE_RX200_ICU_SCI12_SCIX3_MASK	0x00000020U
#endif	/* EE_RX200_SCI12_SCIX3_ISR */

/** RIIC0 Event/Error interrupt. */
#ifdef	EE_RX200_RIIC0_EEI0_ISR
#ifdef	EE_RX200_RIIC0_EEI0_ISR_PRI
#define	EE_RX200_ICU_IPR_RIIC0_EEI0	EE_HWREG8(0x000873F6U)
#endif	/* EE_RX200_RIIC0_EEI0_ISR_PRI */
#define	EE_RX200_ICU_IER_RIIC0_EEI0	EE_HWREG8(0x0008721EU)
#define	EE_RX200_ICU_RIIC0_EEI0_MASK	0x00000040U
#endif	/* EE_RX200_RIIC0_EEI0_ISR */

/** RIIC0 Receive Data Full interrupt. */
#ifdef	EE_RX200_RIIC0_RXI0_ISR
#ifdef	EE_RX200_RIIC0_RXI0_ISR_PRI
#define	EE_RX200_ICU_IPR_RIIC0_RXI0	EE_HWREG8(0x000873F7U)
#endif	/* EE_RX200_RIIC0_RXI0_ISR_PRI */
#define	EE_RX200_ICU_IER_RIIC0_RXI0	EE_HWREG8(0x0008721EU)
#define	EE_RX200_ICU_RIIC0_RXI0_MASK	0x00000080U
#endif	/* EE_RX200_RIIC0_RXI0_ISR */

/** RIIC0 TX End interrupt. */
#ifdef	EE_RX200_RIIC0_TXI0_ISR
#ifdef	EE_RX200_RIIC0_TXI0_ISR_PRI
#define	EE_RX200_ICU_IPR_RIIC0_TXI0	EE_HWREG8(0x000873F8U)
#endif	/* EE_RX200_RIIC0_TXI0_ISR_PRI */
#define	EE_RX200_ICU_IER_RIIC0_TXI0	EE_HWREG8(0x0008721FU)
#define	EE_RX200_ICU_RIIC0_TXI0_MASK	0x00000001U
#endif	/* EE_RX200_RIIC0_TXI0_ISR */

/** RIIC0 TX Data Empty interrupt. */
#ifdef	EE_RX200_RIIC0_TEI0_ISR
#ifdef	EE_RX200_RIIC0_TEI0_ISR_PRI
#define	EE_RX200_ICU_IPR_RIIC0_TEI0	EE_HWREG8(0x000873F9U)
#endif	/* EE_RX200_RIIC0_TEI0_ISR_PRI */
#define	EE_RX200_ICU_IER_RIIC0_TEI0	EE_HWREG8(0x0008721FU)
#define	EE_RX200_ICU_RIIC0_TEI0_MASK	0x00000002U
#endif	/* EE_RX200_RIIC0_TEI0_ISR */

/** SCI11 RX Error interrupt. */
#ifdef	EE_RX200_SCI11_ERI11_ISR
#ifdef	EE_RX200_SCI11_ERI11_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI11_ERI11	EE_HWREG8(0x000873FAU)
#endif	/* EE_RX200_SCI11_ERI11_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI11_ERI11	EE_HWREG8(0x0008721FU)
#define	EE_RX200_ICU_SCI11_ERI11_MASK	0x00000004U
#endif	/* EE_RX200_SCI11_ERI11_ISR */

/** SCI11 RX interrupt. */
#ifdef	EE_RX200_SCI11_RXI11_ISR
#ifdef	EE_RX200_SCI11_RXI11_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI11_RXI11	EE_HWREG8(0x000873FAU)
#endif	/* EE_RX200_SCI11_RXI11_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI11_RXI11	EE_HWREG8(0x0008721FU)
#define	EE_RX200_ICU_SCI11_RXI11_MASK	0x00000008U
#endif	/* EE_RX200_SCI11_RXI11_ISR */

/** SCI11 TX Empty interrupt. */
#ifdef	EE_RX200_SCI11_TXI11_ISR
#ifdef	EE_RX200_SCI11_TXI11_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI11_TXI11	EE_HWREG8(0x000873FAU)
#endif	/* EE_RX200_SCI11_TXI11_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI11_TXI11	EE_HWREG8(0x0008721FU)
#define	EE_RX200_ICU_SCI11_TXI11_MASK	0x00000010U
#endif	/* EE_RX200_SCI11_TXI11_ISR */

/** SCI11 TX End interrupt. */
#ifdef	EE_RX200_SCI11_TEI11_ISR
#ifdef	EE_RX200_SCI11_TEI11_ISR_PRI
#define	EE_RX200_ICU_IPR_SCI11_TEI11	EE_HWREG8(0x000873FAU)
#endif	/* EE_RX200_SCI11_TEI11_ISR_PRI */
#define	EE_RX200_ICU_IER_SCI11_TEI11	EE_HWREG8(0x0008721FU)
#define	EE_RX200_ICU_SCI11_TEI11_MASK	0x00000020U
#endif	/* EE_RX200_SCI11_TEI11_ISR */

#endif /* __INCLUDE_RENESAS_R5F5210X_ISR_H__ */
